/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   SELOR_RQUOTA                                                                  
   Programado por Lucas Priori                                                   
   Versao: 1.00                                                                  
                                                                                 
   Compilar:                                                                     
   Linux: gcc selor_rquota.c -o selor_rquota -lmysqlclient                       
   FreeBSD:  gcc selor_rquota.c -o selor_rquota -DFreeBSD -L/usr/local/lib/mysql -lmysqlclient
                                                                                 
  *******************************************************************            
  Selor_rquota e' um utilitario do pacote Selor para se registrar e controlar o  
  numero de emails enviados de seus clientes                                     
  Copyright (C) 2014 Lucas Priori                                                
                                                                                 
  This program is free software; you can redistribute it and/or                  
  modify it under the terms of the GNU General Public License                    
  as published by the Free Software Foundation; either version 2                 
  of the License, or (at your option) any later version.                         
                                                                                 
  This program is distributed in the hope that it will be useful,                
  but WITHOUT ANY WARRANTY; without even the implied warranty of                 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  
  GNU General Public License for more details.                                   
                                                                                 
  You should have received a copy of the GNU General Public License              
  along with this program; if not, write to the Free Software                    
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#ifdef FreeBSD
 #include "/usr/local/include/mysql/mysql.h"
 #include "/usr/local/include/mysql/errmsg.h"
#else
 #include<mysql/mysql.h>
 #include<mysql/errmsg.h>
#endif

// ----
#define VERSAO "1.0"
#define TAM_buffer 1024
#define TAM_host 150
#define TAM_op 70


// ----
#define HOST "localhost"
#define DB "selor_rquota"
#define TABLE "rquota"
#define USER "selor_user"
#define PASS "selor_pass"
#define USER_FIELD "user"
#define SENT_FIELD "sent"
#define QUOTA_FIELD "quota"
#define QUOTA_default 1000


//---------------------------------------------------
//      Variaveis GLOBAIS                            
//---------------------------------------------------
MYSQL *mysql = NULL;
MYSQL_RES *myres; 
MYSQL_ROW myreg;

// -- Variaveis para conexao com o Banco de dados --
char host[TAM_host]=HOST;
char db[TAM_op]=DB;
char table[TAM_op]=TABLE;
char user[TAM_op]=USER;
char pass[TAM_op]=PASS;
char user_field[TAM_op]=USER_FIELD;
char sent_field[TAM_op]=SENT_FIELD;
char quota_field[TAM_op]=QUOTA_FIELD;
unsigned int quota_default=QUOTA_default;

// -- Variaveis gerais --
char op_conf[200];
char op_c=0;
unsigned int op_a=0;
unsigned int op_d=QUOTA_default;
unsigned int op_z=0;


// ------------------------------------ 
// --------- Funcao Principal ---------
// ------------------------------------
int main(int argc,char *argv[]){
 FILE *arq_conf;
 char *ptr_aux1=NULL;
 char user_op[TAM_buffer];
 char ret1=0;
 char cmd_sql[TAM_buffer + 1];
 char linha[TAM_buffer + 1];
 int num_opcoes=0;
 int b_lidos=0;
 unsigned int quota=0;
 unsigned int cont1=0;
 unsigned int cont2=0;
 unsigned int sent=0;
 
 
 if(argc < 2){
  Ajuda:
  printf("Selor_rquota %s\n", VERSAO);
  printf("You must to specify the command and user\n");
  printf(" Command |  Description\n");
  printf("  -a         Updates the database with the number of recipients sent by sender\n");
  printf("                If the user has not exists, it is created\n");
  printf("                Ex: ./selor_rquota -a 50 -u user@domainofsender.xxx\n");
  printf("                Ex: ./selor_rquota -a 1 -u user@domainofsender.xxx\n");
  printf("                Ex: ./selor_rquota -a 300 -u user@domainofsender.xxx\n");
  printf("  -d         Specifies the user default quota\n");
  printf("                This option must be used with the '-a' option\n");
  printf("                Ex: ./selor_rquota -d 1000 -a 10 -u user@domainofsender.xxx\n");
  printf("  -c         Checks if user has your quota exceeded\n");
  printf("                Ex: ./selor_rquota -c -u user@domainofsender.xxx\n");
  printf("  -u         To specify the user\n");
  printf("  -z         Resets the number of emails sent by all users or\n");
  printf("                only user specified by '-u' option\n");
  printf("  --conf     The configuration file with database connection info [OPTIONAL]\n");
  printf("                These information may be set inside of this program through of source code:\n");
  printf("                The '#defines' are:\n");
  printf("                #define HOST \"localhost\"\n");
  printf("                #define DB \"selor_rquota\"\n");
  printf("                #define TABLE \"rquota\"\n");
  printf("                #define USER \"selor_user\"\n");
  printf("                #define PASS \"selor_pass\"\n");
  printf("                #define USER_FIELD \"user\"\n");
  printf("                #define SENT_FIELD \"sent\"\n");
  printf("                #define QUOTA_FIELD \"quota\"\n");
  printf("                #define QUOTA_default 1000\n");
  printf("\nThis program returns 0(Zero) if the user has not reached the quota or 1 in otherwise\n\n");
  return 0;
 }
 
 user_op[0]=0;
 op_conf[0]=0;
 // -- Lendo os parametros --
 for(cont1=1; cont1<argc; cont1++){
  if(!strcmp(argv[cont1-1], "-a")){
   op_a=atoi(argv[cont1]);
   num_opcoes++;
  }else if(!strcmp(argv[cont1-1], "-d")){
   op_d=atoi(argv[cont1]);
   num_opcoes++;
  }else if(!strcmp(argv[cont1], "-c")){
   op_c=1;
   num_opcoes++;
  }else if(!strcmp(argv[cont1-1], "-u")){
   strncpy(user_op, argv[cont1], TAM_buffer);
  }else if(!strcmp(argv[cont1], "-z")){
   op_z=1;
  }else if(!strcmp(argv[cont1-1], "--conf")){
   snprintf(op_conf, sizeof(op_conf)-1, "%s", argv[cont1]);
  }
 }
 
 if(!op_z && (!num_opcoes || !user_op[0])){
  printf("You have a error in parameter list. See the help\n");
  goto Ajuda;
 }
 
 
 
 if(op_conf[0]){
  // -- Lendo informacoes do arquivo de conexao --
  arq_conf=fopen(op_conf, "rb");
  if(!arq_conf){
   printf("Could not open the configuration file %s\n", arq_conf);
   return 0;
  }
  linha[0]=0;
  cont2=0;
  while(!feof(arq_conf)){
   b_lidos=fread(cmd_sql, 1, TAM_buffer, arq_conf);
   for(cont1=0; cont1<b_lidos; cont1++){
    if(cmd_sql[cont1] == 10){
     linha[cont2]=0;
     if(!strncmp(linha, "Host=", 5)){
      ptr_aux1=&linha[5];
      while(*ptr_aux1 == ' ' || *ptr_aux1 == 9)*ptr_aux1++;
      snprintf(host, TAM_host, "%s", ptr_aux1);
     
     }else if(!strncmp(linha, "Db=", 3)){
      ptr_aux1=&linha[3];
      while(*ptr_aux1 == ' ' || *ptr_aux1 == 9)*ptr_aux1++;
      snprintf(db, TAM_op, "%s", ptr_aux1);
     
     }else if(!strncmp(linha, "Table=", 6)){
      ptr_aux1=&linha[6];
      while(*ptr_aux1 == ' ' || *ptr_aux1 == 9)*ptr_aux1++;
      snprintf(table, TAM_op, "%s", ptr_aux1);
     
     }else if(!strncmp(linha, "User=", 5)){
      ptr_aux1=&linha[5];
      while(*ptr_aux1 == ' ' || *ptr_aux1 == 9)*ptr_aux1++;
      snprintf(user, TAM_op, "%s", ptr_aux1);
     
     }else if(!strncmp(linha, "Pass=", 5)){
      ptr_aux1=&linha[5];
      while(*ptr_aux1 == ' ' || *ptr_aux1 == 9)*ptr_aux1++;
      snprintf(pass, TAM_op, "%s", ptr_aux1);
     
     }else if(!strncmp(linha, "User_field=", 11)){
      ptr_aux1=&linha[11];
      while(*ptr_aux1 == ' ' || *ptr_aux1 == 9)*ptr_aux1++;
      snprintf(user_field, TAM_op, "%s", ptr_aux1);
     
     }else if(!strncmp(linha, "Sent_field=", 11)){
      ptr_aux1=&linha[11];
      while(*ptr_aux1 == ' ' || *ptr_aux1 == 9)*ptr_aux1++;
      snprintf(sent_field, TAM_op, "%s", ptr_aux1);
     
     }else if(!strncmp(linha, "Quota_field=", 12)){
      ptr_aux1=&linha[12];
      while(*ptr_aux1 == ' ' || *ptr_aux1 == 9)*ptr_aux1++;
      snprintf(quota_field, TAM_op, "%s", ptr_aux1);
     
     }else if(!strncmp(linha, "Quota_default=", 14)){
      ptr_aux1=&linha[14];
      while(*ptr_aux1 == ' ' || *ptr_aux1 == 9)*ptr_aux1++;
      quota_default=atoi(ptr_aux1);
     }
     cont2=0;
    }else if(cmd_sql[cont1] != 13)linha[cont2++]=cmd_sql[cont1];
   }
  }
  fclose(arq_conf);
 }
 
 
 // -- Iniciando a lib mysql --
 if(!(mysql = mysql_init(NULL))){
  printf("Could not initialize the MySQL functions\n");
  return 0;  // --> Retornando Zero para no bloquear o usuario
 }
 
 // -- Conectando no Banco de Dados --
 if(mysql_real_connect(mysql, host, user, pass, db, 0, NULL, 0)){
  
  // ------------
  //  Opcao -z   
  // ------------
  if(op_z){
   // -- Zerando os valores do(s) usuario(s) --
   if(!user_op[0])snprintf(cmd_sql, sizeof(cmd_sql), "UPDATE %s SET %s=0", table, sent_field);
   else snprintf(cmd_sql, sizeof(cmd_sql), "UPDATE %s SET %s=0 WHERE %s='%s'", table, sent_field, user_field, user_op);
   mysql_query(mysql, cmd_sql);
   mysql_close(mysql);
   return 0;
  }
  
  // -- Consultando a quota do usuario --
  snprintf(cmd_sql, sizeof(cmd_sql), "SELECT %s,%s FROM %s WHERE %s='%s'", sent_field, quota_field, table, user_field, user_op);
  ret1=0;
  if(!mysql_query(mysql, cmd_sql)){
   myres=mysql_store_result(mysql);
   if((myreg=mysql_fetch_row(myres))){
    sent=atoi(myreg[0]);
    quota=atoi(myreg[1]);
    ret1=1;
   }
   mysql_free_result(myres);
  }else {
   printf("MySQL ERROR in: %s", cmd_sql);
   return 0;             // --> Retornando 0 para nao recusar o email
  }
  if(op_c && ret1){
   if(sent >= quota){
    mysql_close(mysql);           // Quota foi ultrapassada
    return 1;
   }
  }
  if(op_a){
   if(!ret1){
    // -- Usuario nao existe --
    //     Vamos cadatra-lo    
    snprintf(cmd_sql,sizeof(cmd_sql), "INSERT INTO %s (%s,%s,%s) VALUES ('%s','%d','%d')", table, user_field, sent_field, quota_field, user_op, op_a, op_d);
    mysql_query(mysql,cmd_sql); 
   }else {
    snprintf(cmd_sql,sizeof(cmd_sql), "UPDATE %s SET %s=%s+%d WHERE %s='%s'", table, sent_field, sent_field, op_a, user_field, user_op);
    mysql_query(mysql, cmd_sql);
   }
  }
 }else printf("Could not connect to MySQL server\n");
 
 mysql_close(mysql);
 return 0;
}

/*
CREATE TABLE `rquota` (
  `user` varchar(255) NOT NULL default '',
  `sent` int(10) unsigned NOT NULL default '0',
  `quota` int(10) unsigned NOT NULL default '0',
  KEY `user_key` (`user`)
) TYPE=MyISAM;

*/

